#include <animation/sprite.h>
#include <stdlib.h>

#include "raylib.h"
#include "raymath.h"

SpriteAnimation CreateSpriteAnimation(Texture2D texture, Vector2 sprite_texture_dimentions, Vector2 sprite_size, float rotation, float scale, int frameCount) {
    SpriteAnimation spriteAnimation = malloc(sizeof(sprite_animation_t));
    spriteAnimation->texture = texture;
    spriteAnimation->dimentions = sprite_texture_dimentions;
    spriteAnimation->sourceRect = (Rectangle){0, 0, sprite_size.x, sprite_size.y};
    spriteAnimation->size = sprite_size;
    spriteAnimation->rotation = rotation;
    spriteAnimation->scale = scale;
    spriteAnimation->frameCount = frameCount;
    spriteAnimation->currentFrame = 0;
    spriteAnimation->time = 0.f;
    spriteAnimation->speed = 1.f;

    return spriteAnimation;
}

void UpdateSpriteAnimation(SpriteAnimation self, float dt) {
    self->time += dt;
    if (self->time >= self->speed) {
        self->time = 0.f;
        self->currentFrame++;
    }

    if(self->currentFrame >= self->frameCount) {
        self->currentFrame = 0;
    }

    self->sourceRect.x = (self->currentFrame * (int)self->size.x) % (int)self->dimentions.x;
    self->sourceRect.y = (self->currentFrame * (int)self->size.y) % (int)self->dimentions.y;
}

void DrawSpriteAnimation(SpriteAnimation spriteAnimation, Vector2 position) {
    Vector2 drawpos = Vector2Add(position, Vector2Scale(spriteAnimation->size, 2.f));
    DrawTexturePro(
        spriteAnimation->texture,
        (Rectangle){
            spriteAnimation->sourceRect.x,
            spriteAnimation->sourceRect.y,
            spriteAnimation->sourceRect.width,
            spriteAnimation->sourceRect.height
        },
        (Rectangle){
            position.x,
            position.y,
            spriteAnimation->size.x * spriteAnimation->scale,
            spriteAnimation->size.y * spriteAnimation->scale
        },
        position,
        spriteAnimation->rotation,
        WHITE
    );
}